from marshmallow import Schema, fields, post_load
from ..resource import Resource
from collections import namedtuple


class Transaction(Resource):
    """
    https://dev.chartmogul.com/v1.0/reference#transactions
    """
    _path = "/import/invoices{/uuid}/transactions"

    class _Schema(Schema):
        uuid = fields.String()
        external_id = fields.String(allow_none=True)
        type = fields.String()
        date = fields.DateTime()
        result = fields.String()

        @post_load
        def make(self, data):
            return Transaction(**data)

    _schema = _Schema(strict=True)



VERSION = "v1"
API_BASE = "https://api.chartmogul.com"


class Config:
    uri = API_BASE + "/" + VERSION

    def __init__(self, account_token, secret_key):
        self.auth = (account_token, secret_key)



from marshmallow import Schema, fields, post_load
from ..resource import Resource
from collections import namedtuple


class Plan(Resource):
    """
    https://dev.chartmogul.com/v1.0/reference#plans
    """
    _path = "/plans{/uuid}"
    _root_key = 'plans'
    _many = namedtuple('Plans', [_root_key, "current_page", "total_pages"])

    class _Schema(Schema):
        uuid = fields.String()
        data_source_uuid = fields.String()
        name = fields.String()
        interval_count = fields.Int()
        interval_unit = fields.String()
        external_id = fields.String()

        @post_load
        def make(self, data):
            return Plan(**data)

    _schema = _Schema(strict=True)



from marshmallow import Schema, fields, post_load
from ..resource import Resource
from collections import namedtuple


class DataSource(Resource):
    """
    https://dev.chartmogul.com/v1.0/reference#data-sources
    """
    _path = '/data_sources{/uuid}'
    _root_key = 'data_sources'
    _many = namedtuple('DataSources', [_root_key])

    class _Schema(Schema):
        uuid = fields.String()
        name = fields.String()
        created_at = fields.DateTime()
        status = fields.Str()
        system = fields.Str()

        @post_load
        def make(self, data):
            return DataSource(**data)

    _schema = _Schema(strict=True)



from marshmallow import Schema, fields, post_load
from ..resource import Resource, DataObject, _add_method
from .customer import Customer
from collections import namedtuple


class CustomAttributes(Resource):
    """
    https://dev.chartmogul.com/v1.0/reference#custom-attributes
    """
    _path = "/customers{/uuid}/attributes/custom"

    class _Schema(Schema):
        custom = fields.Dict()

        @post_load
        def make(self, data):
            return CustomAttributes(**data)

    _customers = namedtuple('Customers', ['entries'])
    _schema = _Schema(strict=True)

    @classmethod
    def _load(cls, response):
        """
        Workaround: when using email, the return value is actually
        a list of customers, but without paging.
        """
        response.raise_for_status()
        if response.status_code == 204:
            return None
        jsonObj = response.json()
        if 'entries' in jsonObj:
            customers = Customer._schema.load(jsonObj['entries'], many=True).data
            return cls._customers(customers)
        else:
            return cls._schema.load(jsonObj).data



from marshmallow import Schema, fields, post_load
from ..resource import Resource, DataObject


class Stripe(DataObject):
    class _Schema(Schema):
        uid = fields.Int()
        coupon = fields.Boolean()

        @post_load
        def make(self, data):
            return Stripe(**data)


class Name(DataObject):
    class _Schema(Schema):
        fullName = fields.String()

        @post_load
        def make(self, data):
            return Name(**data)


class Employment(DataObject):
    class _Schema(Schema):
        name = fields.String()

        @post_load
        def make(self, data):
            return Employment(**data)


class Person(DataObject):
    class _Schema(Schema):
        name = fields.Nested(Name._Schema)
        employment = fields.Nested(Employment._Schema)

        @post_load
        def make(self, data):
            return Person(**data)


class Company(DataObject):
    class _Schema(Schema):
        name = fields.String()
        legalName = fields.String()
        domain = fields.String()
        url = fields.String()
        category = fields.Dict()
        metrics = fields.Dict()

        @post_load
        def make(self, data):
            return Company(**data)


class Clearbit(DataObject):
    class _Schema(Schema):
        company = fields.Nested(Company._Schema)
        person = fields.Nested(Person._Schema)

        @post_load
        def make(self, data):
            return Clearbit(**data)


class Attributes(Resource):
    """
    https://dev.chartmogul.com/v1.0/reference#customer-attributes
    """
    _path = "/customers{/uuid}/attributes"

    class _Schema(Schema):
        tags = fields.List(fields.String())
        stripe = fields.Nested(Stripe._Schema)
        clearbit = fields.Nested(Clearbit._Schema)
        custom = fields.Dict()

        @post_load
        def make(self, data):
            return Attributes(**data)

    _schema = _Schema(strict=True)



from marshmallow import Schema, fields, post_load
from ..resource import Resource, DataObject
from .transaction import Transaction
from collections import namedtuple


class LineItem(DataObject):

    class _Schema(Schema):
        uuid = fields.String()
        external_id = fields.String(allow_none=True)
        type = fields.String()
        subscription_uuid = fields.String()
        subscription_external_id = fields.String()
        plan_uuid = fields.String()
        prorated = fields.Boolean()
        service_period_start = fields.DateTime()
        service_period_end = fields.DateTime()
        amount_in_cents = fields.Int()
        quantity = fields.Int()
        discount_code = fields.String(allow_none=True)
        discount_amount_in_cents = fields.Int()
        tax_amount_in_cents = fields.Int()
        account_code = fields.String(allow_none=True)

        @post_load
        def make(self, data):
            return LineItem(**data)


class Invoice(Resource):
    """
    https://dev.chartmogul.com/v1.0/reference#invoices
    """
    _path = "/import/customers{/uuid}/invoices"
    _root_key = 'invoices'
    _many = namedtuple('Invoices', [_root_key, "current_page", "total_pages", "customer_uuid"])
    _many.__new__.__defaults__ = (None,) * len(_many._fields)

    class _Schema(Schema):
        uuid = fields.String()
        external_id = fields.String(allow_none=True)
        date = fields.DateTime()
        due_date = fields.DateTime()
        currency = fields.String()
        line_items = fields.Nested(LineItem._Schema, many=True)
        transactions = fields.Nested(Transaction._Schema, many=True)

        @post_load
        def make(self, data):
            return Invoice(**data)

    _schema = _Schema(strict=True)
